var SpeedDialUtils = {
  prefs: null,
  thumbFolder: "SDThumbs",
  cacheService: null,

  getPrefs: function() {
    if (SpeedDialUtils.prefs == null) {
      SpeedDialUtils.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch2);
    }
    return SpeedDialUtils.prefs;
  },
  
  getCacheService: function() {
    if (SpeedDialUtils.cacheService == null) {
      SpeedDialUtils.cacheService = Components.classes["@uworks.net/speeddialcache;1"].getService().wrappedJSObject;
    }
    return SpeedDialUtils.cacheService;
  },
  
  removeThumbnailImage: function(thumbNumber) {
    var prefs = SpeedDialUtils.getPrefs();
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
                       .getService(Components.interfaces.nsIProperties)
                       .get("ProfD", Components.interfaces.nsIFile);
    file.append(SpeedDialUtils.thumbFolder);
    
    // Delete all files that match this thumbnail!
    var rootNameMatch = "thumbnail-" + thumbNumber + "(-\\d+)?\\.(png|jpg)";
    var enumerator = file.directoryEntries;
    while (enumerator.hasMoreElements()) {
      var targetFile = enumerator.getNext().QueryInterface(Components.interfaces.nsIFile);
      if ((targetFile.isFile()) && (targetFile.leafName.match(rootNameMatch))) {
        try {
          targetFile.remove(false);
        } catch(e) {
          // Do nothing...
        }
      }
    }
  },

  clearThumbnailData: function(thumbNumber) {
    var prefs = SpeedDialUtils.getPrefs();

    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + thumbNumber + "-url")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + thumbNumber + "-url");
    }

    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + thumbNumber + "-icon")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + thumbNumber + "-icon");
    }

    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + thumbNumber + "-label")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + thumbNumber + "-label");
    }

    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + thumbNumber + "-lastsaved")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + thumbNumber + "-lastsaved");
    }

    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + thumbNumber + "-dynamictitle")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + thumbNumber + "-dynamictitle");
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + thumbNumber + "-refreshinterval")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + thumbNumber + "-refreshinterval");
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + thumbNumber + "-manualrefresh")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + thumbNumber + "-manualrefresh");
    }

    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + thumbNumber + "-format")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + thumbNumber + "-format");
    }

    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + thumbNumber + "-backgroundcolor")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + thumbNumber + "-backgroundcolor");
    }

    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + thumbNumber + "-layout")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + thumbNumber + "-layout");
    }

    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + thumbNumber + "-cropping")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + thumbNumber + "-cropping");
    }

    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + thumbNumber + "-thumbnailurl")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + thumbNumber + "-thumbnailurl");
    }
  },
  
  deassignThumbnail: function(thumbNumber) {
    // Take out from cache
    SpeedDialUtils.getCacheService().setImage(null, thumbNumber);

    SpeedDialUtils.removeThumbnailImage(thumbNumber);
    
    SpeedDialUtils.clearThumbnailData(thumbNumber);
  },
  
  removeGroup: function(groupIndex) {
    try {
    var prefs = SpeedDialUtils.getPrefs();

    var speedDialBundle = document.getElementById("bundle_speeddial");
    
    if (prefs.getBoolPref("extensions.speeddial.reorderDialsAfterGroupDelete") &&
        prefs.getBoolPref("extensions.speeddial.askBeforeRemovingGroup") &&
        speedDialBundle) {
      var ps = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                                .getService(Components.interfaces.nsIPromptService);
      var dontAskAgain= { value: false };
      var rv = ps.confirmEx(window,
                            speedDialBundle.getString("removewarning.title"),
                            speedDialBundle.getString("removewarning.message"),
                            Components.interfaces.nsIPromptService.STD_YES_NO_BUTTONS,
                            null, null, null,
                            speedDialBundle.getString("removewarning.dontask"),
                            dontAskAgain);
      if (dontAskAgain.value) {
        prefs.setBoolPref("extensions.speeddial.askBeforeRemovingGroup", false);
      }
      if (rv != 0) return;
    }
    
    var totalGroups = prefs.getIntPref("extensions.speeddial.numGroups");
    
    if (totalGroups < 2) return;
    
    // Move data
    for (var c=groupIndex; c<totalGroups; c++) {
      // Move title
      if (prefs.prefHasUserValue("extensions.speeddial.group-" + (c+1) + "-title")) {
        prefs.setComplexValue("extensions.speeddial.group-" + c + "-title", Components.interfaces.nsISupportsString, prefs.getComplexValue("extensions.speeddial.group-" + (c+1) + "-title", Components.interfaces.nsISupportsString));
        prefs.clearUserPref("extensions.speeddial.group-" + (c+1) + "-title");
      } else if (prefs.prefHasUserValue("extensions.speeddial.group-" + c + "-title")) {
        prefs.clearUserPref("extensions.speeddial.group-" + c + "-title");
      }
      
      // Move background color
      if (prefs.prefHasUserValue("extensions.speeddial.group-" + (c+1) + "-backgroundColor")) {
        prefs.setCharPref("extensions.speeddial.group-" + c + "-backgroundColor", prefs.getCharPref("extensions.speeddial.group-" + (c+1) + "-backgroundColor"));
        prefs.clearUserPref("extensions.speeddial.group-" + (c+1) + "-backgroundColor");
      } else if (prefs.prefHasUserValue("extensions.speeddial.group-" + c + "-backgroundColor")) {
        prefs.clearUserPref("extensions.speeddial.group-" + c + "-backgroundColor");
      }
      
      // Move text color
      if (prefs.prefHasUserValue("extensions.speeddial.group-" + (c+1) + "-textColor")) {
        prefs.setCharPref("extensions.speeddial.group-" + c + "-textColor", prefs.getCharPref("extensions.speeddial.group-" + (c+1) + "-textColor"));
        prefs.clearUserPref("extensions.speeddial.group-" + (c+1) + "-textColor");
      } else if (prefs.prefHasUserValue("extensions.speeddial.group-" + c + "-textColor")) {
        prefs.clearUserPref("extensions.speeddial.group-" + c + "-textColor");
      }
      
    }
    
    if (prefs.getBoolPref("extensions.speeddial.reorderDialsAfterGroupDelete")) {
      var thumbsPerGroup = prefs.getIntPref("extensions.speeddial.columns") * 
                           prefs.getIntPref("extensions.speeddial.rows");
      for (var c=(groupIndex-1)*thumbsPerGroup; c<totalGroups*thumbsPerGroup; c++) {
        SpeedDialUtils.deassignThumbnail(c);
        SpeedDialUtils.removeThumbnailImage(c);
        if (c<((totalGroups - 1)*thumbsPerGroup)) {
          SpeedDialUtils.copyThumbnailData(c+thumbsPerGroup, c);
          SpeedDialUtils.moveThumbnailImage(c+thumbsPerGroup, c);
        }
      }

        
//      for (var c=(totalGroups - 1)*thumbsPerGroup; c<totalGroups*thumbsPerGroup; c++) {
//        SpeedDialUtils.deassignThumbnail(c);
//        SpeedDialUtils.removeThumbnailImage(c);
//      }
    }
    
    prefs.setIntPref("extensions.speeddial.numGroups", totalGroups - 1);
    } catch (e) { alert(e); }
  },

  swapGroup: function(firstGroup, secondGroup) {
    var prefs = SpeedDialUtils.getPrefs();

    // Swap title
    if (prefs.prefHasUserValue("extensions.speeddial.group-" + firstGroup + "-title")) {
      if (prefs.prefHasUserValue("extensions.speeddial.group-" + secondGroup + "-title")) {
        // Swap
        var groupTitle = prefs.getComplexValue("extensions.speeddial.group-" + firstGroup + "-title", Components.interfaces.nsISupportsString);
        prefs.setComplexValue("extensions.speeddial.group-" + firstGroup + "-title", Components.interfaces.nsISupportsString, prefs.getComplexValue("extensions.speeddial.group-" + secondGroup + "-title", Components.interfaces.nsISupportsString));
        prefs.setComplexValue("extensions.speeddial.group-" + secondGroup + "-title", Components.interfaces.nsISupportsString, groupTitle);
      } else {
        // Move first to second
        prefs.setComplexValue("extensions.speeddial.group-" + secondGroup + "-title", Components.interfaces.nsISupportsString, prefs.getComplexValue("extensions.speeddial.group-" + firstGroup + "-title", Components.interfaces.nsISupportsString));
        prefs.clearUserPref("extensions.speeddial.group-" + firstGroup + "-title");
      }
    } else if (prefs.prefHasUserValue("extensions.speeddial.group-" + secondGroup + "-title")) {
      // Move second to first
      prefs.setComplexValue("extensions.speeddial.group-" + firstGroup + "-title", Components.interfaces.nsISupportsString, prefs.getComplexValue("extensions.speeddial.group-" + secondGroup + "-title", Components.interfaces.nsISupportsString));
      prefs.clearUserPref("extensions.speeddial.group-" + secondGroup + "-title");
    }

    // Swap background color
    if (prefs.prefHasUserValue("extensions.speeddial.group-" + firstGroup + "-backgroundColor")) {
      if (prefs.prefHasUserValue("extensions.speeddial.group-" + secondGroup + "-backgroundColor")) {
        // Swap
        var groupBackgroundColor = prefs.getCharPref("extensions.speeddial.group-" + firstGroup + "-backgroundColor");
        prefs.setCharPref("extensions.speeddial.group-" + firstGroup + "-backgroundColor", prefs.getCharPref("extensions.speeddial.group-" + secondGroup + "-backgroundColor"));
        prefs.setCharPref("extensions.speeddial.group-" + secondGroup + "-backgroundColor", groupBackgroundColor);
      } else {
        // Move first to second
        prefs.setCharPref("extensions.speeddial.group-" + secondGroup + "-backgroundColor", prefs.getCharPref("extensions.speeddial.group-" + firstGroup + "-backgroundColor"));
        prefs.clearUserPref("extensions.speeddial.group-" + firstGroup + "-backgroundColor");
      }
    } else if (prefs.prefHasUserValue("extensions.speeddial.group-" + secondGroup + "-backgroundColor")) {
      // Move second to first
      prefs.setCharPref("extensions.speeddial.group-" + firstGroup + "-backgroundColor", prefs.getCharPref("extensions.speeddial.group-" + secondGroup + "-backgroundColor"));
      prefs.clearUserPref("extensions.speeddial.group-" + secondGroup + "-backgroundColor");
    }
    
    // Swap text color
    if (prefs.prefHasUserValue("extensions.speeddial.group-" + firstGroup + "-textColor")) {
      if (prefs.prefHasUserValue("extensions.speeddial.group-" + secondGroup + "-textColor")) {
        // Swap
        var groupTextColor = prefs.getCharPref("extensions.speeddial.group-" + firstGroup + "-textColor");
        prefs.setCharPref("extensions.speeddial.group-" + firstGroup + "-textColor", prefs.getCharPref("extensions.speeddial.group-" + secondGroup + "-textColor"));
        prefs.setCharPref("extensions.speeddial.group-" + secondGroup + "-textColor", groupTextColor);
      } else {
        // Move first to second
        prefs.setCharPref("extensions.speeddial.group-" + secondGroup + "-textColor", prefs.getCharPref("extensions.speeddial.group-" + firstGroup + "-textColor"));
        prefs.clearUserPref("extensions.speeddial.group-" + firstGroup + "-textColor");
      }
    } else if (prefs.prefHasUserValue("extensions.speeddial.group-" + secondGroup + "-textColor")) {
      // Move second to first
      prefs.setCharPref("extensions.speeddial.group-" + firstGroup + "-textColor", prefs.getCharPref("extensions.speeddial.group-" + secondGroup + "-textColor"));
      prefs.clearUserPref("extensions.speeddial.group-" + secondGroup + "-textColor");
    }
    
    var thumbsPerGroup = prefs.getIntPref("extensions.speeddial.columns") * 
                         prefs.getIntPref("extensions.speeddial.rows");
    for (var c=0; c<thumbsPerGroup; c++) {
      SpeedDialUtils.swapThumbnails(((firstGroup - 1) * thumbsPerGroup + c + 1), ((secondGroup - 1) * thumbsPerGroup + c + 1));
    }
  },

  copyThumbnailData: function(originalDial, targetDial) {
    var prefs = SpeedDialUtils.getPrefs();

    // Copy data
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalDial + "-url")) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + targetDial + "-url", prefs.getCharPref("extensions.speeddial.thumbnail-" + originalDial + "-url"));
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalDial + "-icon")) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + targetDial + "-icon", prefs.getCharPref("extensions.speeddial.thumbnail-" + originalDial + "-icon"));
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalDial + "-label")) {
      prefs.setComplexValue("extensions.speeddial.thumbnail-" + targetDial + "-label", Components.interfaces.nsISupportsString, prefs.getComplexValue("extensions.speeddial.thumbnail-" + originalDial + "-label", Components.interfaces.nsISupportsString));
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalDial + "-lastsaved")) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + targetDial + "-lastsaved", prefs.getCharPref("extensions.speeddial.thumbnail-" + originalDial + "-lastsaved"));
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalDial + "-dynamictitle")) {
      prefs.setBoolPref("extensions.speeddial.thumbnail-" + targetDial + "-dynamictitle", prefs.getBoolPref("extensions.speeddial.thumbnail-" + originalDial + "-dynamictitle"));
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalDial + "-refreshinterval")) {
      prefs.setIntPref("extensions.speeddial.thumbnail-" + targetDial + "-refreshinterval", prefs.getIntPref("extensions.speeddial.thumbnail-" + originalDial + "-refreshinterval"));
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalDial + "-manualrefresh")) {
      prefs.setBoolPref("extensions.speeddial.thumbnail-" + targetDial + "-manualrefresh", prefs.getBoolPref("extensions.speeddial.thumbnail-" + originalDial + "-manualrefresh"));
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalDial + "-format")) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + targetDial + "-format", prefs.getCharPref("extensions.speeddial.thumbnail-" + originalDial + "-format"));
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalDial + "-backgroundcolor")) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + targetDial + "-backgroundcolor", prefs.getCharPref("extensions.speeddial.thumbnail-" + originalDial + "-backgroundcolor"));
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalDial + "-layout")) {
      prefs.setIntPref("extensions.speeddial.thumbnail-" + targetDial + "-layout", prefs.getIntPref("extensions.speeddial.thumbnail-" + originalDial + "-layout"));
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalDial + "-cropping")) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + targetDial + "-cropping", prefs.getCharPref("extensions.speeddial.thumbnail-" + originalDial + "-cropping"));
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalDial + "-thumbnailurl")) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + targetDial + "-thumbnailurl", prefs.getCharPref("extensions.speeddial.thumbnail-" + originalDial + "-thumbnailurl"));
    }
  },

  copyThumbnailImage: function(originalNumber, destinationNumber) {
    var prefs = SpeedDialUtils.getPrefs();
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
                       .getService(Components.interfaces.nsIProperties)
                       .get("ProfD", Components.interfaces.nsIFile);
    file.append(SpeedDialUtils.thumbFolder);
    var thumbFolder = file.clone();
    
    var fileExtension = prefs.getCharPref("extensions.speeddial.imageFormat");
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalNumber + "-format")) {
      fileExtension = prefs.getCharPref("extensions.speeddial.thumbnail-" + originalNumber + "-format");
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + destinationNumber + "-format")) {
      fileExtension = prefs.getCharPref("extensions.speeddial.thumbnail-" + destinationNumber + "-format");
    }  
    
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalNumber + "-lastsaved")) {
      file.append("thumbnail-" + originalNumber + "-" + prefs.getCharPref("extensions.speeddial.thumbnail-" + originalNumber + "-lastsaved") + "." + fileExtension);
    } else {
      file.append("thumbnail-" + originalNumber + "." + fileExtension);
    }
    
    var destFileName;
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + destinationNumber + "-lastsaved")) {
      destFileName = "thumbnail-" + destinationNumber + "-" + prefs.getCharPref("extensions.speeddial.thumbnail-" + destinationNumber + "-lastsaved") + "." + fileExtension;
    } else {
      destFileName = "thumbnail-" + destinationNumber + "." + fileExtension;
    }
    
    if (file.exists() && !file.isDirectory()) {
      file.copyTo(thumbFolder, destFileName);
    }
  },

  moveThumbnailImage: function(originalNumber, destinationNumber) {
    var prefs = SpeedDialUtils.getPrefs();
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
                       .getService(Components.interfaces.nsIProperties)
                       .get("ProfD", Components.interfaces.nsIFile);
    file.append(SpeedDialUtils.thumbFolder);
    var thumbFolder = file.clone();
  
    var fileExtension = prefs.getCharPref("extensions.speeddial.imageFormat");
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalNumber + "-format")) {
      fileExtension = prefs.getCharPref("extensions.speeddial.thumbnail-" + originalNumber + "-format");
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + destinationNumber + "-format")) {
      fileExtension = prefs.getCharPref("extensions.speeddial.thumbnail-" + destinationNumber + "-format");
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + originalNumber + "-lastsaved")) {
      file.append("thumbnail-" + originalNumber + "-" + prefs.getCharPref("extensions.speeddial.thumbnail-" + originalNumber + "-lastsaved") + "." + fileExtension);
    } else {
      file.append("thumbnail-" + originalNumber + "." + fileExtension);
    }
  
    var destFileName;
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + destinationNumber + "-lastsaved")) {
      destFileName = "thumbnail-" + destinationNumber + "-" + prefs.getCharPref("extensions.speeddial.thumbnail-" + destinationNumber + "-lastsaved") + "." + fileExtension;
    } else {
      destFileName = "thumbnail-" + destinationNumber + "." + fileExtension;
    }
  //alert("file: " + file.path + ", to: " + destFileName);
    if (file.exists() && !file.isDirectory()) {
      file.moveTo(thumbFolder, destFileName);
    }
  },
  
  swapThumbnailImage: function(firstNumber, secondNumber) {
    var prefs = SpeedDialUtils.getPrefs();
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
                       .getService(Components.interfaces.nsIProperties)
                       .get("ProfD", Components.interfaces.nsIFile);
    file.append(SpeedDialUtils.thumbFolder);
    var thumbFolder = file.clone();
  
    var firstFileExtension = prefs.getCharPref("extensions.speeddial.imageFormat");
    var secondFileExtension = firstFileExtension;
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstNumber + "-format")) {
      firstFileExtension = prefs.getCharPref("extensions.speeddial.thumbnail-" + firstNumber + "-format");
    }
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondNumber + "-format")) {
      secondFileExtension = prefs.getCharPref("extensions.speeddial.thumbnail-" + secondNumber + "-format");
    }
    
    var firstFile = file.clone();
    var secondFile = file.clone();
    var firstFileName;
    var secondFileName;
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstNumber + "-lastsaved")) {
      var lastSaved = prefs.getCharPref("extensions.speeddial.thumbnail-" + firstNumber + "-lastsaved");
      firstFile.append("thumbnail-" + firstNumber + "-" + lastSaved + "." + firstFileExtension);
      secondFileName = "thumbnail-" + secondNumber + "-" + lastSaved + "." + firstFileExtension;
    } else {
      firstFile.append("thumbnail-" + firstNumber + "." + firstFileExtension);
      secondFileName = "thumbnail-" + secondNumber + "." + firstFileExtension;
    }
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondNumber + "-lastsaved")) {
      var lastSaved = prefs.getCharPref("extensions.speeddial.thumbnail-" + secondNumber + "-lastsaved");
      secondFile.append("thumbnail-" + secondNumber + "-" + lastSaved + "." + secondFileExtension);
      firstFileName = "thumbnail-" + firstNumber + "-" + lastSaved + "." + secondFileExtension;
    } else {
      secondFile.append("thumbnail-" + secondNumber + "." + secondFileExtension);
      firstFileName = "thumbnail-" + firstNumber + "." + secondFileExtension;
    }
    
    var tempFileName = "thumbnail.tmp";
    file.append(tempFileName);
    
    if (!firstFile.exists() || firstFile.isDirectory()) {
      if (secondFile.exists() && !secondFile.isDirectory()) {
        secondFile.moveTo(thumbFolder, firstFileName);
      }
    } else if (!secondFile.exists() || secondFile.isDirectory()) {
      firstFile.moveTo(thumbFolder, secondFileName);
    } else {
      firstFile.moveTo(thumbFolder, tempFileName);
      secondFile.moveTo(thumbFolder, firstFileName);
      file.moveTo(thumbFolder, secondFileName);
    }
    
    // Swap cache, also
    var cacheService = SpeedDialUtils.getCacheService();
    var firstCache = cacheService.getImage(firstNumber);
    cacheService.setImageObject(cacheService.getImage(secondNumber), firstNumber);
    cacheService.setImageObject(firstCache, secondNumber);
  },
  
  swapThumbnails: function(firstDial, secondDial) {
    var prefs = SpeedDialUtils.getPrefs();

    var dialURL = null;
    var dialIcon = null;
    var dialLabel = null;
    var dialLastSaved = null;
    var dialDynamicTitle = null;
    var dialRefreshInterval = null;
    var dialManualRefresh = null;
    var dialFormat = null;
    var dialBackgroundColor = null;
    var dialLayout = null;
    var dialCropping = null;
    var dialThumbnailURL = null;

    // Copy data
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstDial + "-url")) {
      dialURL = prefs.getCharPref("extensions.speeddial.thumbnail-" + firstDial + "-url");
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstDial + "-icon")) {
      dialIcon = prefs.getCharPref("extensions.speeddial.thumbnail-" + firstDial + "-icon");
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstDial + "-label")) {
      dialLabel = prefs.getComplexValue("extensions.speeddial.thumbnail-" + firstDial + "-label", Components.interfaces.nsISupportsString);
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstDial + "-lastsaved")) {
      dialLastSaved = prefs.getCharPref("extensions.speeddial.thumbnail-" + firstDial + "-lastsaved");
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstDial + "-dynamictitle")) {
      dialDynamicTitle = prefs.getBoolPref("extensions.speeddial.thumbnail-" + firstDial + "-dynamictitle");
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstDial + "-refreshinterval")) {
      dialRefreshInterval = prefs.getIntPref("extensions.speeddial.thumbnail-" + firstDial + "-refreshinterval");
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstDial + "-manualrefresh")) {
      dialManualRefresh = prefs.getBoolPref("extensions.speeddial.thumbnail-" + firstDial + "-manualrefresh");
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstDial + "-format")) {
      dialFormat = prefs.getCharPref("extensions.speeddial.thumbnail-" + firstDial + "-format");
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstDial + "-backgroundcolor")) {
      dialBackgroundColor = prefs.getCharPref("extensions.speeddial.thumbnail-" + firstDial + "-backgroundcolor");
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstDial + "-layout")) {
      dialLayout = prefs.getIntPref("extensions.speeddial.thumbnail-" + firstDial + "-layout");
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstDial + "-cropping")) {
      dialCropping = prefs.getCharPref("extensions.speeddial.thumbnail-" + firstDial + "-cropping");
    }
  
    if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + firstDial + "-thumbnailurl")) {
      dialThumbnailURL = prefs.getCharPref("extensions.speeddial.thumbnail-" + firstDial + "-thumbnailurl");
    }
    
    // Swap thumbnails
    SpeedDialUtils.swapThumbnailImage(secondDial, firstDial);
    
    // Clean first dial
    SpeedDialUtils.clearThumbnailData(firstDial);

    SpeedDialUtils.copyThumbnailData(secondDial, firstDial);
    
    if (dialURL != null) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + secondDial + "-url", dialURL);
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondDial + "-url")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + secondDial + "-url");
    }
    if (dialIcon != null) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + secondDial + "-icon", dialIcon);
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondDial + "-icon")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + secondDial + "-icon");
    }
    if (dialLabel != null) {
      prefs.setComplexValue("extensions.speeddial.thumbnail-" + secondDial + "-label", Components.interfaces.nsISupportsString, dialLabel);
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondDial + "label")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + secondDial + "-label");
    }
    if (dialLastSaved != null) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + secondDial + "-lastsaved", dialLastSaved);
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondDial + "-lastsaved")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + secondDial + "-lastsaved");
    }
    if (dialDynamicTitle != null) {
      prefs.setBoolPref("extensions.speeddial.thumbnail-" + secondDial + "-dynamictitle", dialDynamicTitle);
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondDial + "-dynamictitle")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + secondDial + "-dynamictitle");
    }
    if (dialRefreshInterval != null) {
      prefs.setIntPref("extensions.speeddial.thumbnail-" + secondDial + "-refreshinterval", dialRefreshInterval);
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondDial + "-refreshinterval")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + secondDial + "-refreshinterval");
    }
    if (dialManualRefresh != null) {
      prefs.setBoolPref("extensions.speeddial.thumbnail-" + secondDial + "-manualrefresh", dialManualRefresh);
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondDial + "-manualrefresh")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + secondDial + "-manualrefresh");
    }
    if (dialFormat != null) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + secondDial + "-format", dialFormat);
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondDial + "-format")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + secondDial + "-format");
    }
    if (dialBackgroundColor != null) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + secondDial + "-backgroundcolor", dialBackgroundColor);
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondDial + "-backgroundcolor")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + secondDial + "-backgroundcolor");
    }
    if (dialLayout != null) {
      prefs.setIntPref("extensions.speeddial.thumbnail-" + secondDial + "-layout", dialLayout);
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondDial + "-layout")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + secondDial + "-layout");
    }
    if (dialCropping != null) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + secondDial + "-cropping", dialCropping);
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondDial + "-cropping")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + secondDial + "-cropping");
    }
    if (dialThumbnailURL != null) {
      prefs.setCharPref("extensions.speeddial.thumbnail-" + secondDial + "-thumbnailurl", dialThumbnailURL);
    } else if (prefs.prefHasUserValue("extensions.speeddial.thumbnail-" + secondDial + "-thumbnailurl")) {
      prefs.clearUserPref("extensions.speeddial.thumbnail-" + secondDial + "-thumbnailurl");
    }
  },
  
  getDomain: function(url) {
    var result = url.split("/");
    if (result.length < 3) {
      return "";
    }
    
    try {
      var domain = result[2].toLowerCase();
      var resultDot = domain.split(".");
      if (resultDot.length > 1) {
        return resultDot[resultDot.length - 2] + "." + resultDot[resultDot.length - 1];
      } else {
        return domain;
      }
    } catch (e) {
      return "";
    }
  }


  
}